{-# OPTIONS_GHC -fno-warn-unused-binds -fno-warn-missing-signatures #-}
{-# LANGUAGE CPP #-}
{-# LINE 1 "NewIfLexer.x" #-}

{-

Open Source Fixedpoint Model-Checker version 2024

(C) Copyright Sebastian Moedersheim 2003,2024
(C) Copyright Jakub Janaszkiewicz 2022
(C) Copyright Paolo Modesti 2012
(C) Copyright Nicklas Bo Jensen 2012
(C) Copyright IBM Corp. 2009
(C) Copyright ETH Zurich (Swiss Federal Institute of Technology) 2003,2007

All Rights Reserved.

-}


module NewIfLexer (Token(..), PIdent, AlexPosn(..), alexScanTokens
                  , token_posn
                  ) where

#if __GLASGOW_HASKELL__ >= 603
#include "ghcconfig.h"
#elif defined(__GLASGOW_HASKELL__)
#include "config.h"
#endif
#if __GLASGOW_HASKELL__ >= 503
import Data.Array
import Data.Array.Base (unsafeAt)
#else
import Array
#endif
{-# LINE 1 "templates/wrappers.hs" #-}
-- -----------------------------------------------------------------------------
-- Alex wrapper code.
--
-- This code is in the PUBLIC DOMAIN; you may copy it freely and use
-- it for any purpose whatsoever.






import Data.Word (Word8)
















import Data.Char (ord)
import qualified Data.Bits

-- | Encode a Haskell String to a list of Word8 values, in UTF8 format.
utf8Encode :: Char -> [Word8]
utf8Encode = map fromIntegral . go . ord
 where
  go oc
   | oc <= 0x7f       = [oc]

   | oc <= 0x7ff      = [ 0xc0 + (oc `Data.Bits.shiftR` 6)
                        , 0x80 + oc Data.Bits..&. 0x3f
                        ]

   | oc <= 0xffff     = [ 0xe0 + (oc `Data.Bits.shiftR` 12)
                        , 0x80 + ((oc `Data.Bits.shiftR` 6) Data.Bits..&. 0x3f)
                        , 0x80 + oc Data.Bits..&. 0x3f
                        ]
   | otherwise        = [ 0xf0 + (oc `Data.Bits.shiftR` 18)
                        , 0x80 + ((oc `Data.Bits.shiftR` 12) Data.Bits..&. 0x3f)
                        , 0x80 + ((oc `Data.Bits.shiftR` 6) Data.Bits..&. 0x3f)
                        , 0x80 + oc Data.Bits..&. 0x3f
                        ]



type Byte = Word8

-- -----------------------------------------------------------------------------
-- The input type


type AlexInput = (AlexPosn,     -- current position,
                  Char,         -- previous char
                  [Byte],       -- pending bytes on current char
                  String)       -- current input string

ignorePendingBytes :: AlexInput -> AlexInput
ignorePendingBytes (p,c,_ps,s) = (p,c,[],s)

alexInputPrevChar :: AlexInput -> Char
alexInputPrevChar (_p,c,_bs,_s) = c

alexGetByte :: AlexInput -> Maybe (Byte,AlexInput)
alexGetByte (p,c,(b:bs),s) = Just (b,(p,c,bs,s))
alexGetByte (_,_,[],[]) = Nothing
alexGetByte (p,_,[],(c:s))  = let p' = alexMove p c
                                  (b:bs) = utf8Encode c
                              in p' `seq`  Just (b, (p', c, bs, s))





























































-- -----------------------------------------------------------------------------
-- Token positions

-- `Posn' records the location of a token in the input text.  It has three
-- fields: the address (number of chacaters preceding the token), line number
-- and column of a token within the file. `start_pos' gives the position of the
-- start of the file and `eof_pos' a standard encoding for the end of file.
-- `move_pos' calculates the new position after traversing a given character,
-- assuming the usual eight character tab stops.


data AlexPosn = AlexPn !Int !Int !Int
        deriving (Eq,Show)

alexStartPos :: AlexPosn
alexStartPos = AlexPn 0 1 1

alexMove :: AlexPosn -> Char -> AlexPosn
alexMove (AlexPn a l c) '\t' = AlexPn (a+1)  l     (((c+alex_tab_size-1) `div` alex_tab_size)*alex_tab_size+1)
alexMove (AlexPn a l _) '\n' = AlexPn (a+1) (l+1)   1
alexMove (AlexPn a l c) _    = AlexPn (a+1)  l     (c+1)


-- -----------------------------------------------------------------------------
-- Default monad
















































































































-- -----------------------------------------------------------------------------
-- Monad (with ByteString input)







































































































-- -----------------------------------------------------------------------------
-- Basic wrapper

























-- -----------------------------------------------------------------------------
-- Basic wrapper, ByteString version
































-- -----------------------------------------------------------------------------
-- Posn wrapper

-- Adds text positions to the basic model.


--alexScanTokens :: String -> [token]
alexScanTokens str0 = go (alexStartPos,'\n',[],str0)
  where go inp__@(pos,_,_,str) =
          case alexScan inp__ 0 of
                AlexEOF -> []
                AlexError ((AlexPn _ line column),_,_,_) -> error $ "lexical error at line " ++ (show line) ++ ", column " ++ (show column)
                AlexSkip  inp__' _ln     -> go inp__'
                AlexToken inp__' len act -> act pos (take len str) : go inp__'



-- -----------------------------------------------------------------------------
-- Posn wrapper, ByteString version














-- -----------------------------------------------------------------------------
-- GScan wrapper

-- For compatibility with previous versions of Alex, and because we can.














alex_tab_size :: Int
alex_tab_size = 8
alex_base :: Array Int Int
alex_base = listArray (0 :: Int, 162)
  [ -8
  , -41
  , -86
  , -33
  , -36
  , -72
  , 0
  , 41
  , 0
  , 169
  , 297
  , -52
  , -6
  , 553
  , 489
  , 0
  , 3
  , 735
  , 0
  , 0
  , 0
  , -51
  , 0
  , 0
  , 0
  , 0
  , 0
  , -28
  , 0
  , 0
  , 701
  , 0
  , -44
  , 0
  , 0
  , 0
  , 0
  , 0
  , 702
  , 0
  , 0
  , 703
  , 943
  , 1018
  , 1093
  , 1168
  , 1243
  , 1318
  , 1393
  , 1468
  , 0
  , 1543
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 1618
  , 1693
  , 1768
  , 0
  , 1843
  , 0
  , 1918
  , 1993
  , 2068
  , 2143
  , 2218
  , 2293
  , 2368
  , 2443
  , 2518
  , 2593
  , 2668
  , 2743
  , 2818
  , 2893
  , 2968
  , 3043
  , 3118
  , 3193
  , 3268
  , 3343
  , 3418
  , 3493
  , 3568
  , 3643
  , 3718
  , 3793
  , 3868
  , 3943
  , 4018
  , 4093
  , 4168
  , 4243
  , 4318
  , 4393
  , 4468
  , 4543
  , 4618
  , 4693
  , 4768
  , 4843
  , 4918
  , 4993
  , 5068
  , 5143
  , 5218
  , 5293
  , 5368
  , 5443
  , 5518
  , 5593
  , 5668
  , 5743
  , 5818
  , 5893
  , 5968
  , 6043
  , 6118
  , 6193
  , 6268
  , 6343
  , 6418
  , 6493
  , 6568
  , 6643
  , 6718
  , 6793
  , 6868
  , 6943
  , 7018
  , 7093
  , 7168
  , 7243
  , 7318
  , 7393
  , 7468
  , 7543
  , 7618
  , 7693
  , 7768
  , 7843
  , 7918
  , 7993
  , 8068
  , 8143
  , 8218
  , 8293
  , 8368
  , 8443
  , 8518
  , 8593
  , 8668
  , 8743
  , 8818
  , 8893
  , 8968
  , 9043
  , 9118
  , 9193
  , 778
  ]

alex_table :: Array Int Int
alex_table = listArray (0 :: Int, 9448)
  [ 0
  , 16
  , 16
  , 16
  , 16
  , 16
  , 20
  , 24
  , 26
  , 11
  , 25
  , 33
  , 16
  , 16
  , 16
  , 16
  , 16
  , 3
  , 31
  , 0
  , 0
  , 0
  , 0
  , 0
  , 16
  , 0
  , 0
  , 0
  , 0
  , 17
  , 35
  , 0
  , 27
  , 28
  , 34
  , 16
  , 37
  , 21
  , 36
  , 12
  , 162
  , 162
  , 162
  , 162
  , 162
  , 162
  , 162
  , 162
  , 162
  , 162
  , 30
  , 0
  , 4
  , 32
  , 23
  , 40
  , 0
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 38
  , 1
  , 39
  , 19
  , 0
  , 0
  , 144
  , 119
  , 119
  , 119
  , 97
  , 119
  , 92
  , 70
  , 127
  , 119
  , 119
  , 86
  , 119
  , 73
  , 158
  , 126
  , 119
  , 104
  , 140
  , 103
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 22
  , 14
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 5
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 10
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 13
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 63
  , 2
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 29
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 18
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 162
  , 162
  , 162
  , 162
  , 162
  , 162
  , 162
  , 162
  , 162
  , 162
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 13
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 15
  , 14
  , 10
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 6
  , 5
  , 9
  , 8
  , 8
  , 8
  , 7
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 149
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 148
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 41
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 43
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 46
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 89
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 51
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 59
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 60
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 62
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 120
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 160
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 65
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 159
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 66
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 156
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 155
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 154
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 153
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 152
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 116
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 151
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 150
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 147
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 146
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 145
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 71
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 72
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 74
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 76
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 77
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 78
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 81
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 82
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 142
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 141
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 83
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 84
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 85
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 96
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 139
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 138
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 137
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 95
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 135
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 134
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 133
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 132
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 131
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 101
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 129
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 128
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 105
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 106
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 107
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 110
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 125
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 113
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 124
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 123
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 115
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 122
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 118
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 117
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 112
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 111
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 109
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 108
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 130
  , 100
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 99
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 98
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 136
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 94
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 93
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 91
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 90
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 88
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 87
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 80
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 79
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 143
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 102
  , 119
  , 119
  , 119
  , 114
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 75
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 69
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 68
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 67
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 157
  , 119
  , 119
  , 119
  , 119
  , 119
  , 121
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 64
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 61
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 58
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 57
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 56
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 55
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 54
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 53
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 52
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 50
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 49
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 48
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 47
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 45
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 44
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 0
  , 0
  , 0
  , 0
  , 119
  , 0
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 42
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 119
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 0
  , 0
  , 0
  , 0
  , 161
  , 0
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 161
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  ]

alex_check :: Array Int Int
alex_check = listArray (0 :: Int, 9448)
  [ -1
  , 9
  , 10
  , 11
  , 12
  , 13
  , 47
  , 93
  , 41
  , 45
  , 62
  , 62
  , 9
  , 10
  , 11
  , 12
  , 13
  , 45
  , 62
  , -1
  , -1
  , -1
  , -1
  , -1
  , 32
  , -1
  , -1
  , -1
  , -1
  , 37
  , 38
  , -1
  , 40
  , 41
  , 42
  , 32
  , 44
  , 45
  , 46
  , 47
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , 58
  , -1
  , 60
  , 61
  , 62
  , 61
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , 91
  , 92
  , 93
  , 92
  , -1
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , -1
  , -1
  , -1
  , 126
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 143
  , 144
  , 145
  , 146
  , 147
  , 148
  , 149
  , 150
  , 151
  , 152
  , 153
  , 154
  , 155
  , 156
  , 157
  , 158
  , 159
  , 160
  , 161
  , 162
  , 163
  , 164
  , 165
  , 166
  , 167
  , 168
  , 169
  , 170
  , 171
  , 172
  , 173
  , 174
  , 175
  , 176
  , 177
  , 178
  , 179
  , 180
  , 181
  , 182
  , 183
  , 184
  , 185
  , 186
  , 187
  , 188
  , 189
  , 190
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 128
  , 129
  , 130
  , 131
  , 132
  , 133
  , 134
  , 135
  , 136
  , 137
  , 138
  , 139
  , 140
  , 141
  , 142
  , 143
  , 144
  , 145
  , 146
  , 147
  , 148
  , 149
  , 150
  , 151
  , 152
  , 153
  , 154
  , 155
  , 156
  , 157
  , 158
  , 159
  , 160
  , 161
  , 162
  , 163
  , 164
  , 165
  , 166
  , 167
  , 168
  , 169
  , 170
  , 171
  , 172
  , 173
  , 174
  , 175
  , 176
  , 177
  , 178
  , 179
  , 180
  , 181
  , 182
  , 183
  , 184
  , 185
  , 186
  , 187
  , 188
  , 189
  , 190
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 128
  , 129
  , 130
  , 131
  , 132
  , 133
  , 134
  , 135
  , 136
  , 137
  , 138
  , 139
  , 140
  , 141
  , 142
  , 143
  , 144
  , 145
  , 146
  , 147
  , 148
  , 149
  , 150
  , 151
  , 152
  , 153
  , 154
  , 155
  , 156
  , 157
  , 158
  , 159
  , 160
  , 161
  , 162
  , 163
  , 164
  , 165
  , 166
  , 167
  , 168
  , 169
  , 170
  , 171
  , 172
  , 173
  , 174
  , 175
  , 176
  , 177
  , 178
  , 179
  , 180
  , 181
  , 182
  , 183
  , 184
  , 185
  , 186
  , 187
  , 188
  , 189
  , 190
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 0
  , 1
  , 2
  , 3
  , 4
  , 5
  , 6
  , 7
  , 8
  , 9
  , 10
  , 11
  , 12
  , 13
  , 14
  , 15
  , 16
  , 17
  , 18
  , 19
  , 20
  , 21
  , 22
  , 23
  , 24
  , 25
  , 26
  , 27
  , 28
  , 29
  , 30
  , 31
  , 32
  , 33
  , 34
  , 35
  , 36
  , 37
  , 38
  , 39
  , 40
  , 41
  , 42
  , 43
  , 44
  , 45
  , 46
  , 47
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , 58
  , 59
  , 60
  , 61
  , 62
  , 63
  , 64
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , 91
  , 92
  , 93
  , 94
  , 95
  , 96
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 123
  , 124
  , 125
  , 126
  , 127
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 10
  , 45
  , 45
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , 61
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , 93
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 128
  , 129
  , 130
  , 131
  , 132
  , 133
  , 134
  , 135
  , 136
  , 137
  , 138
  , 139
  , 140
  , 141
  , 142
  , 143
  , 144
  , 145
  , 146
  , 147
  , 148
  , 149
  , 150
  , 151
  , 152
  , 153
  , 154
  , 155
  , 156
  , 157
  , 158
  , 159
  , 160
  , 161
  , 162
  , 163
  , 164
  , 165
  , 166
  , 167
  , 168
  , 169
  , 170
  , 171
  , 172
  , 173
  , 174
  , 175
  , 176
  , 177
  , 178
  , 179
  , 180
  , 181
  , 182
  , 183
  , 184
  , 185
  , 186
  , 187
  , 188
  , 189
  , 190
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , 58
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , 58
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , 58
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , 58
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , 58
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , 58
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , 58
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , 58
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  ]

alex_deflt :: Array Int Int
alex_deflt = listArray (0 :: Int, 162)
  [ -1
  , -1
  , -1
  , -1
  , -1
  , 15
  , 15
  , 6
  , 6
  , -1
  , -1
  , -1
  , -1
  , 17
  , 17
  , 17
  , -1
  , 17
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  ]

alex_accept = listArray (0 :: Int, 162)
  [ AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccSkip
  , AlexAccSkip
  , AlexAcc 144
  , AlexAcc 143
  , AlexAcc 142
  , AlexAcc 141
  , AlexAcc 140
  , AlexAcc 139
  , AlexAcc 138
  , AlexAcc 137
  , AlexAcc 136
  , AlexAcc 135
  , AlexAcc 134
  , AlexAcc 133
  , AlexAcc 132
  , AlexAcc 131
  , AlexAcc 130
  , AlexAcc 129
  , AlexAcc 128
  , AlexAcc 127
  , AlexAcc 126
  , AlexAcc 125
  , AlexAcc 124
  , AlexAcc 123
  , AlexAcc 122
  , AlexAcc 121
  , AlexAcc 120
  , AlexAcc 119
  , AlexAcc 118
  , AlexAcc 117
  , AlexAcc 116
  , AlexAcc 115
  , AlexAcc 114
  , AlexAcc 113
  , AlexAcc 112
  , AlexAcc 111
  , AlexAcc 110
  , AlexAcc 109
  , AlexAcc 108
  , AlexAcc 107
  , AlexAcc 106
  , AlexAcc 105
  , AlexAcc 104
  , AlexAcc 103
  , AlexAcc 102
  , AlexAcc 101
  , AlexAcc 100
  , AlexAcc 99
  , AlexAcc 98
  , AlexAcc 97
  , AlexAcc 96
  , AlexAcc 95
  , AlexAcc 94
  , AlexAcc 93
  , AlexAcc 92
  , AlexAcc 91
  , AlexAcc 90
  , AlexAcc 89
  , AlexAcc 88
  , AlexAcc 87
  , AlexAcc 86
  , AlexAcc 85
  , AlexAcc 84
  , AlexAcc 83
  , AlexAcc 82
  , AlexAcc 81
  , AlexAcc 80
  , AlexAcc 79
  , AlexAcc 78
  , AlexAcc 77
  , AlexAcc 76
  , AlexAcc 75
  , AlexAcc 74
  , AlexAcc 73
  , AlexAcc 72
  , AlexAcc 71
  , AlexAcc 70
  , AlexAcc 69
  , AlexAcc 68
  , AlexAcc 67
  , AlexAcc 66
  , AlexAcc 65
  , AlexAcc 64
  , AlexAcc 63
  , AlexAcc 62
  , AlexAcc 61
  , AlexAcc 60
  , AlexAcc 59
  , AlexAcc 58
  , AlexAcc 57
  , AlexAcc 56
  , AlexAcc 55
  , AlexAcc 54
  , AlexAcc 53
  , AlexAcc 52
  , AlexAcc 51
  , AlexAcc 50
  , AlexAcc 49
  , AlexAcc 48
  , AlexAcc 47
  , AlexAcc 46
  , AlexAcc 45
  , AlexAcc 44
  , AlexAcc 43
  , AlexAcc 42
  , AlexAcc 41
  , AlexAcc 40
  , AlexAcc 39
  , AlexAcc 38
  , AlexAcc 37
  , AlexAcc 36
  , AlexAcc 35
  , AlexAcc 34
  , AlexAcc 33
  , AlexAcc 32
  , AlexAcc 31
  , AlexAcc 30
  , AlexAcc 29
  , AlexAcc 28
  , AlexAcc 27
  , AlexAcc 26
  , AlexAcc 25
  , AlexAcc 24
  , AlexAcc 23
  , AlexAcc 22
  , AlexAcc 21
  , AlexAcc 20
  , AlexAcc 19
  , AlexAcc 18
  , AlexAcc 17
  , AlexAcc 16
  , AlexAcc 15
  , AlexAcc 14
  , AlexAcc 13
  , AlexAcc 12
  , AlexAcc 11
  , AlexAcc 10
  , AlexAcc 9
  , AlexAcc 8
  , AlexAcc 7
  , AlexAcc 6
  , AlexAcc 5
  , AlexAcc 4
  , AlexAcc 3
  , AlexAcc 2
  , AlexAcc 1
  , AlexAcc 0
  ]

alex_actions = array (0 :: Int, 145)
  [ (144,alex_action_2)
  , (143,alex_action_3)
  , (142,alex_action_4)
  , (141,alex_action_5)
  , (140,alex_action_6)
  , (139,alex_action_7)
  , (138,alex_action_8)
  , (137,alex_action_9)
  , (136,alex_action_10)
  , (135,alex_action_11)
  , (134,alex_action_12)
  , (133,alex_action_13)
  , (132,alex_action_14)
  , (131,alex_action_15)
  , (130,alex_action_16)
  , (129,alex_action_17)
  , (128,alex_action_18)
  , (127,alex_action_19)
  , (126,alex_action_20)
  , (125,alex_action_21)
  , (124,alex_action_22)
  , (123,alex_action_23)
  , (122,alex_action_24)
  , (121,alex_action_25)
  , (120,alex_action_26)
  , (119,alex_action_27)
  , (118,alex_action_28)
  , (117,alex_action_29)
  , (116,alex_action_30)
  , (115,alex_action_31)
  , (114,alex_action_32)
  , (113,alex_action_33)
  , (112,alex_action_34)
  , (111,alex_action_35)
  , (110,alex_action_36)
  , (109,alex_action_37)
  , (108,alex_action_38)
  , (107,alex_action_39)
  , (106,alex_action_40)
  , (105,alex_action_41)
  , (104,alex_action_42)
  , (103,alex_action_43)
  , (102,alex_action_44)
  , (101,alex_action_45)
  , (100,alex_action_46)
  , (99,alex_action_47)
  , (98,alex_action_48)
  , (97,alex_action_48)
  , (96,alex_action_48)
  , (95,alex_action_48)
  , (94,alex_action_48)
  , (93,alex_action_48)
  , (92,alex_action_48)
  , (91,alex_action_48)
  , (90,alex_action_48)
  , (89,alex_action_48)
  , (88,alex_action_48)
  , (87,alex_action_48)
  , (86,alex_action_48)
  , (85,alex_action_48)
  , (84,alex_action_48)
  , (83,alex_action_48)
  , (82,alex_action_48)
  , (81,alex_action_48)
  , (80,alex_action_48)
  , (79,alex_action_48)
  , (78,alex_action_48)
  , (77,alex_action_48)
  , (76,alex_action_48)
  , (75,alex_action_48)
  , (74,alex_action_48)
  , (73,alex_action_48)
  , (72,alex_action_48)
  , (71,alex_action_48)
  , (70,alex_action_48)
  , (69,alex_action_48)
  , (68,alex_action_48)
  , (67,alex_action_48)
  , (66,alex_action_48)
  , (65,alex_action_48)
  , (64,alex_action_48)
  , (63,alex_action_48)
  , (62,alex_action_48)
  , (61,alex_action_48)
  , (60,alex_action_48)
  , (59,alex_action_48)
  , (58,alex_action_48)
  , (57,alex_action_48)
  , (56,alex_action_48)
  , (55,alex_action_48)
  , (54,alex_action_48)
  , (53,alex_action_48)
  , (52,alex_action_48)
  , (51,alex_action_48)
  , (50,alex_action_48)
  , (49,alex_action_48)
  , (48,alex_action_48)
  , (47,alex_action_48)
  , (46,alex_action_48)
  , (45,alex_action_48)
  , (44,alex_action_48)
  , (43,alex_action_48)
  , (42,alex_action_48)
  , (41,alex_action_48)
  , (40,alex_action_48)
  , (39,alex_action_48)
  , (38,alex_action_48)
  , (37,alex_action_48)
  , (36,alex_action_48)
  , (35,alex_action_48)
  , (34,alex_action_48)
  , (33,alex_action_48)
  , (32,alex_action_48)
  , (31,alex_action_48)
  , (30,alex_action_48)
  , (29,alex_action_48)
  , (28,alex_action_48)
  , (27,alex_action_48)
  , (26,alex_action_48)
  , (25,alex_action_48)
  , (24,alex_action_48)
  , (23,alex_action_48)
  , (22,alex_action_48)
  , (21,alex_action_48)
  , (20,alex_action_48)
  , (19,alex_action_48)
  , (18,alex_action_48)
  , (17,alex_action_48)
  , (16,alex_action_48)
  , (15,alex_action_48)
  , (14,alex_action_48)
  , (13,alex_action_48)
  , (12,alex_action_48)
  , (11,alex_action_48)
  , (10,alex_action_48)
  , (9,alex_action_48)
  , (8,alex_action_48)
  , (7,alex_action_48)
  , (6,alex_action_48)
  , (5,alex_action_48)
  , (4,alex_action_48)
  , (3,alex_action_48)
  , (2,alex_action_48)
  , (1,alex_action_49)
  , (0,alex_action_50)
  ]

{-# LINE 84 "NewIfLexer.x" #-}


type PIdent=String

data Token= 
            TCONST AlexPosn PIdent	-- 
	    |TVAR AlexPosn PIdent	-- 
	    |TOPENB AlexPosn		-- (
	    |TCLOSEB AlexPosn		-- )
	    |TOPENSQB AlexPosn		-- (
	    |TCLOSESQB AlexPosn		-- )
	    |TEQ AlexPosn		-- =
            |TDEF   AlexPosn             -- :=
	    |TCOLON AlexPosn		-- :
	    |TIMPL AlexPosn		-- =>
            |TSTAR AlexPosn             -- star
	    |TARROW AlexPosn            --  ->
	    |TAND AlexPosn		-- &
	    |TDOT AlexPosn		-- .
	    |TCOMMA AlexPosn		-- ,
	    |TEQUAL AlexPosn		-- equal
	    |TLESS AlexPosn		-- less
	    |TNOT AlexPosn		-- not
	    |TOR AlexPosn		-- or
	    |TIMPLIES AlexPosn		-- implies
	    |TIMPLIES2 AlexPosn		-- another implies
	    |TSTEP AlexPosn		-- step
	    |TSECTION AlexPosn		-- section
            |TEXISTS AlexPosn
            |TSECTYPES AlexPosn
            |TSECSIG AlexPosn
	    |TSECINITS AlexPosn
	    |TSECPROP AlexPosn
	    |TSECRULES AlexPosn
	    |TSECGOALS AlexPosn
	    |TGOAL AlexPosn	  	--
	    |TATTACK AlexPosn	  	--
	    |TINIT  AlexPosn		-- initial_state
            |TGLOBALLY AlexPosn
	    |TGT AlexPosn
	    |TCONJ AlexPosn
	    |TDISJ AlexPosn
	    |TNEG AlexPosn
	    |TBWGLOBALLY AlexPosn
	    |TBWEVENTUALLY AlexPosn
            |TPROP AlexPosn
	    |TPREVIOUS AlexPosn
            |TUNEQUAL AlexPosn
	    |THORNCLAUSES AlexPosn
	    | THC AlexPosn
           deriving (Eq,Show)

token_posn (TUNEQUAL p  )= p
token_posn (TCONST p _  )= p 
token_posn (TVAR p _	)= p
token_posn (TOPENB p	)= p
token_posn (TCLOSEB p	)= p
token_posn (TOPENSQB p	)= p
token_posn (TCLOSESQB p	)= p
token_posn (TEQ p	)= p
token_posn (TDEF   p    )= p
token_posn (TCOLON p	)= p
token_posn (TIMPL p	)= p
token_posn (TSTAR p     )= p
token_posn (TARROW p    )= p
token_posn (TAND p	)= p
token_posn (TDOT p	)= p
token_posn (TGT p)       = p
token_posn (TCOMMA p	)= p
token_posn (TEQUAL p	)= p
token_posn (TLESS p	)= p
token_posn (TNOT p	)= p
token_posn (TOR p	)= p
token_posn (TSTEP p	)= p
token_posn (TSECTION p	)= p
token_posn (TEXISTS p	)= p
token_posn (TSECTYPES p	)= p
token_posn (TSECSIG p	)= p
token_posn (TSECINITS p	)= p
token_posn (TSECPROP p	)= p
token_posn (TSECRULES p	)= p
token_posn (TSECGOALS p	)= p
token_posn (TGOAL p	)= p
token_posn (TATTACK p	)= p
token_posn (TINIT  p	)= p
token_posn (TGLOBALLY p ) = p
token_posn (TCONJ p ) = p
token_posn (TDISJ p ) = p
token_posn (TNEG p ) = p
token_posn (TBWGLOBALLY p ) = p
token_posn (TBWEVENTUALLY p ) = p
token_posn (TPROP p ) = p
token_posn (TPREVIOUS p) = p  
token_posn (THORNCLAUSES p) = p
token_posn (THC p)=p
token_posn (TIMPLIES p)=p
token_posn (TIMPLIES2 p)=p


alex_action_2 =  (\ p s -> TGLOBALLY p)
alex_action_3 =  (\ p s -> TCONJ p)
alex_action_4 =  (\ p s -> TDISJ p)
alex_action_5 =  (\ p s -> TNEG p)
alex_action_6 =  (\ p s -> TNEG p)
alex_action_7 =  (\ p s -> TGT p)
alex_action_8 =  (\ p s -> TBWGLOBALLY p)
alex_action_9 =  (\ p s -> TBWEVENTUALLY p)
alex_action_10 =  (\ p s -> TPREVIOUS p)
alex_action_11 =  (\ p s -> TOPENB p) 
alex_action_12 =  (\ p s -> TCLOSEB p) 
alex_action_13 =  (\ p s -> TDEF p) 
alex_action_14 =  (\ p s -> TCOLON p) 
alex_action_15 =  (\ p s -> TIMPL p) 
alex_action_16 =  (\ p s -> TEQ p) 
alex_action_17 =  (\ p s -> TARROW p) 
alex_action_18 =  (\ p s -> TSTAR p) 
alex_action_19 =  (\ p s -> TAND p) 
alex_action_20 =  (\ p s -> TDOT p) 
alex_action_21 =  (\ p s -> TCOMMA p) 
alex_action_22 =  (\ p s -> TOPENSQB p) 
alex_action_23 =  (\ p s -> TCLOSESQB p) 
alex_action_24 =  (\ p s -> TUNEQUAL p)
alex_action_25 =  (\ p s -> TEXISTS p) 
alex_action_26 =  (\ p s -> TEQUAL p) 
alex_action_27 =  (\ p s -> TLESS p) 
alex_action_28 =  (\ p s -> TNOT p) 
alex_action_29 =  (\ p s -> TOR p) 
alex_action_30 =  (\ p s -> TIMPLIES2 p) 
alex_action_31 =  (\ p s -> TAND p) 
alex_action_32 =  (\ p s -> TSTEP p) 
alex_action_33 =  (\ p s -> TSECTION p) 
alex_action_34 =  (\ p s -> TSECPROP p) 
alex_action_35 =  (\ p s -> TPROP p) 
alex_action_36 =  (\ p s -> TSECTYPES p) 
alex_action_37 =  (\ p s -> TSECSIG p) 
alex_action_38 =  (\ p s -> TSECINITS p) 
alex_action_39 =  (\ p s -> TSECRULES p) 
alex_action_40 =  (\ p s -> TSECGOALS p) 
alex_action_41 =  (\ p s -> TSECGOALS p) 
alex_action_42 =  (\ p s -> TGOAL p) 
alex_action_43 =  (\ p s -> TATTACK p) 
alex_action_44 =  (\ p s -> TINIT p) 
alex_action_45 =  (\ p s -> THORNCLAUSES p)
alex_action_46 =  (\ p s -> THC p)
alex_action_47 =  (\ p s -> TIMPLIES p)
alex_action_48 =  (\ p s -> TCONST p s) 
alex_action_49 =  (\ p s -> TVAR p s) 
alex_action_50 =  (\ p s -> TCONST p s) 
{-# LINE 1 "templates/GenericTemplate.hs" #-}
-- -----------------------------------------------------------------------------
-- ALEX TEMPLATE
--
-- This code is in the PUBLIC DOMAIN; you may copy it freely and use
-- it for any purpose whatsoever.

-- -----------------------------------------------------------------------------
-- INTERNALS and main scanner engine































































alexIndexInt16OffAddr arr off = arr ! off




















alexIndexInt32OffAddr arr off = arr ! off











quickIndex arr i = arr ! i


-- -----------------------------------------------------------------------------
-- Main lexing routines

data AlexReturn a
  = AlexEOF
  | AlexError  !AlexInput
  | AlexSkip   !AlexInput !Int
  | AlexToken  !AlexInput !Int a

-- alexScan :: AlexInput -> StartCode -> AlexReturn a
alexScan input__ (sc)
  = alexScanUser undefined input__ (sc)

alexScanUser user__ input__ (sc)
  = case alex_scan_tkn user__ input__ (0) input__ sc AlexNone of
  (AlexNone, input__') ->
    case alexGetByte input__ of
      Nothing ->



                                   AlexEOF
      Just _ ->



                                   AlexError input__'

  (AlexLastSkip input__'' len, _) ->



    AlexSkip input__'' len

  (AlexLastAcc k input__''' len, _) ->



    AlexToken input__''' len (alex_actions ! k)


-- Push the input through the DFA, remembering the most recent accepting
-- state it encountered.

alex_scan_tkn user__ orig_input len input__ s last_acc =
  input__ `seq` -- strict in the input
  let
  new_acc = (check_accs (alex_accept `quickIndex` (s)))
  in
  new_acc `seq`
  case alexGetByte input__ of
     Nothing -> (new_acc, input__)
     Just (c, new_input) ->



      case fromIntegral c of { (ord_c) ->
        let
                base   = alexIndexInt32OffAddr alex_base s
                offset = (base + ord_c)
                check  = alexIndexInt16OffAddr alex_check offset

                new_s = if (offset >= (0)) && (check == ord_c)
                          then alexIndexInt16OffAddr alex_table offset
                          else alexIndexInt16OffAddr alex_deflt s
        in
        case new_s of
            (-1) -> (new_acc, input__)
                -- on an error, we want to keep the input *before* the
                -- character that failed, not after.
            _ -> alex_scan_tkn user__ orig_input (if c < 0x80 || c >= 0xC0 then (len + (1)) else len)
                                                -- note that the length is increased ONLY if this is the 1st byte in a char encoding)
                        new_input new_s new_acc
      }
  where
        check_accs (AlexAccNone) = last_acc
        check_accs (AlexAcc a  ) = AlexLastAcc a input__ (len)
        check_accs (AlexAccSkip) = AlexLastSkip  input__ (len)

        check_accs (AlexAccPred a predx rest)
           | predx user__ orig_input (len) input__
           = AlexLastAcc a input__ (len)
           | otherwise
           = check_accs rest
        check_accs (AlexAccSkipPred predx rest)
           | predx user__ orig_input (len) input__
           = AlexLastSkip input__ (len)
           | otherwise
           = check_accs rest


data AlexLastAcc
  = AlexNone
  | AlexLastAcc !Int !AlexInput !Int
  | AlexLastSkip     !AlexInput !Int

data AlexAcc user
  = AlexAccNone
  | AlexAcc Int
  | AlexAccSkip

  | AlexAccPred Int (AlexAccPred user) (AlexAcc user)
  | AlexAccSkipPred (AlexAccPred user) (AlexAcc user)

type AlexAccPred user = user -> AlexInput -> Int -> AlexInput -> Bool

-- -----------------------------------------------------------------------------
-- Predicates on a rule

alexAndPred p1 p2 user__ in1 len in2
  = p1 user__ in1 len in2 && p2 user__ in1 len in2

--alexPrevCharIsPred :: Char -> AlexAccPred _
alexPrevCharIs c _ input__ _ _ = c == alexInputPrevChar input__

alexPrevCharMatches f _ input__ _ _ = f (alexInputPrevChar input__)

--alexPrevCharIsOneOfPred :: Array Char Bool -> AlexAccPred _
alexPrevCharIsOneOf arr _ input__ _ _ = arr ! alexInputPrevChar input__

--alexRightContext :: Int -> AlexAccPred _
alexRightContext (sc) user__ _ _ input__ =
     case alex_scan_tkn user__ input__ (0) input__ sc AlexNone of
          (AlexNone, _) -> False
          _ -> True
        -- TODO: there's no need to find the longest
        -- match when checking the right context, just
        -- the first match will do.

